\name{Deriv-package}
\alias{Deriv-package}
\docType{package}
\title{
Symbolic Differentiation
}
\description{
R already contains two differentiation functions: D and deriv.

R's existing functions have several limitations.
Limitations include:
\itemize{
 \item The derivatives table can't be modified at runtime, and is only available
in C.
 \item Function cannot substitute function calls.  eg:
      f <- function(x, y) x + y; deriv(~f(x, x^2), "x")
}

Here are the advantages this implementation:

\itemize{
 \item It is entirely written in R, so would be easier to maintain.
 \item Can differentiate function calls:
      - if the function is in the derivative table, then the chain rule
      is applied.  For example, if you declared that the derivative of
      sin is cos, then it would figure out how to call cos correctly.
      - if the function is not in the derivative table (or it is anonymous),
      then the function body is substituted in.
      - these two methods can be mixed.  An entry in the derivative table
      need not be self-contained -- you don't need to provide an infinite
      chain of derivatives.
 \item It's easy to add custom entries to the derivatives table, e.g.
      drule[["cos"]] <- list(quote(-._d1*sin(._1)))
 \item The output can be an executable function, which makes it suitable for use in
optimization problems.
}
}

\details{
\tabular{ll}{
Package: \tab Deriv\cr
Type: \tab Package\cr
Version: \tab 2.0\cr
Date: \tab 2015-02-05\cr
License: \tab GPL (>= 3)\cr
}
The package contains two useful functions: Deriv() for differentiating
expressions, Simplify() for simplifying symbolically.
}

\author{
Andrew Clausen, Serguei Sokol

Maintainer: Serguei Sokol (sokol at insa-toulouse.fr)
}
\references{
\url{https://andrewclausen.net/computing/deriv.html}
}
\keyword{ package }
\seealso{
\code{\link{D}}, \code{\link{deriv}}, package Ryacas
}
\examples{
\dontrun{f <- function(x) x^2}
\dontrun{Deriv(f)}
# function (x)
# 2 * x
}

